#pragma once

#include "ByteUtils.h"

#include <vector>
#include <iostream>

class ZlibData
{
public:
    void setByte(unsigned idx, unsigned char data)
    {
        mData[idx] = data;
    }

    void setDataSize(std::size_t size)
    {
        mData = std::vector<unsigned char>(size);
    }

    void setCompressionMethod(unsigned char method)
    {
        std::cout << "Got compression input " << static_cast<int>(method) << std::endl;
        mCmf = method;
        mCompressionMethod = ByteUtils::getLowerNBits(method, 4);
        mCompressionInfo = ByteUtils::getHigherNBits(method, 4);

        std::cout << "Got compression method " << static_cast<int>(mCompressionMethod) << " and info " << static_cast<int>(mCompressionInfo) << std::endl;
    }

    void setExtraFlags(unsigned char extraFlags)
    {
        std::cout << "Got flags  " << static_cast<int>(extraFlags) << std::endl;

        mFlg = extraFlags;
        mFlagCheck = ByteUtils::getLowerNBits(extraFlags, 5);
        mFlagDict = ByteUtils::getBitN(extraFlags, 5);
        mFlagLevel = ByteUtils::getHigherNBits(extraFlags, 2);

        std::cout << "Got flag check " << static_cast<int>(mFlagCheck) << " and dict " << static_cast<int>(mFlagDict) << " and level " << static_cast<int>(mFlagLevel) << std::endl;
    }

    void processData()
    {
        unsigned char NO_COMPRESSION = 0x00;
        unsigned char FIXED_HUFFMAN = 0x01;
        unsigned char DYNAMIC_HUFFMAN = 0x02;
        unsigned char ERROR = 0x03;

        bool in_final_block = false;
        unsigned working_byte_id = 0;
        for (unsigned idx=0; idx<mData.size(); idx++)
        {
            auto working_byte = mData[working_byte_id];
            std::cout << "Into process data, byte is: " << static_cast<int>(working_byte) << std::endl;

            auto final_block = ByteUtils::getBitN(working_byte, 0);
            if (final_block)
            {
                std::cout << "Got final block" << std::endl;
                in_final_block = true;
            }

            auto compress_type = ByteUtils::getTwoBitsAtN(working_byte, 1);
            std::cout << "Compress type byte is: " << static_cast<int>(compress_type) << std::endl;

            if (compress_type == NO_COMPRESSION)
            {
                std::cout << "Got NO_COMPRESSION" << std::endl;
            }
            else if (compress_type == FIXED_HUFFMAN)
            {
                std::cout << "Got FIXED_HUFFMAN" << std::endl;
            }
            else if (compress_type == DYNAMIC_HUFFMAN)
            {
                std::cout << "Got DYNAMIC_HUFFMAN" << std::endl;
            }
            else if (compress_type == ERROR)
            {
                std::cout << "Got ERROR" << std::endl;
            }
            break;
        }
    }

private:
    std::vector<unsigned char> mData;
    unsigned char mCmf{0};
    unsigned char mFlg{0};
    unsigned char mCompressionMethod{0};
    unsigned char mCompressionInfo{0};
    unsigned char mFlagCheck{0};
    unsigned char mFlagDict{0};
    unsigned char mFlagLevel{0};
    unsigned char mCheckValue{0};
};
